-- Test campsite_type
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(56);

set search_path to camper, public;

select has_table('campsite_type');
select has_pk('campsite_type' );
select table_privs_are('campsite_type', 'guest', array['SELECT']);
select table_privs_are('campsite_type', 'employee', array['SELECT']);
select table_privs_are('campsite_type', 'admin', array['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('campsite_type', 'authenticator', array[]::text[]);

select has_sequence('campsite_type_campsite_type_id_seq');
select sequence_privs_are('campsite_type_campsite_type_id_seq', 'guest', array[]::text[]);
select sequence_privs_are('campsite_type_campsite_type_id_seq', 'employee', array[]::text[]);
select sequence_privs_are('campsite_type_campsite_type_id_seq', 'admin', array['USAGE']);
select sequence_privs_are('campsite_type_campsite_type_id_seq', 'authenticator', array[]::text[]);

select has_column('campsite_type', 'campsite_type_id');
select col_is_pk('campsite_type', 'campsite_type_id');
select col_type_is('campsite_type', 'campsite_type_id', 'integer');
select col_not_null('campsite_type', 'campsite_type_id');
select col_has_default('campsite_type', 'campsite_type_id');
select col_default_is('campsite_type', 'campsite_type_id', 'nextval(''campsite_type_campsite_type_id_seq''::regclass)');

select has_column('campsite_type', 'company_id');
select col_is_fk('campsite_type', 'company_id');
select fk_ok('campsite_type', 'company_id', 'company', 'company_id');
select col_type_is('campsite_type', 'company_id', 'integer');
select col_not_null('campsite_type', 'company_id');
select col_hasnt_default('campsite_type', 'company_id');

select has_column('campsite_type', 'slug');
select col_is_unique('campsite_type', 'slug');
select col_type_is('campsite_type', 'slug', 'uuid');
select col_not_null('campsite_type', 'slug');
select col_has_default('campsite_type', 'slug');
select col_default_is('campsite_type', 'slug', 'gen_random_uuid()');

select has_column('campsite_type', 'name');
select col_type_is('campsite_type', 'name', 'text');
select col_not_null('campsite_type', 'name');
select col_hasnt_default('campsite_type', 'name');

select has_column('campsite_type', 'description');
select col_type_is('campsite_type', 'description', 'xml');
select col_not_null('campsite_type', 'description');
select col_has_default('campsite_type', 'description');
--select col_default_is('campsite_type', 'description', '');

select has_column('campsite_type', 'active');
select col_type_is('campsite_type', 'active', 'boolean');
select col_not_null('campsite_type', 'active');
select col_has_default('campsite_type', 'active');
select col_default_is('campsite_type', 'active', 'true');


set client_min_messages to warning;
truncate campsite_type cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into campsite_type (company_id, name)
values (2, 'Wooden lodge')
     , (4, 'Bungalow')
;

prepare campsite_type_data as
select company_id, name
from campsite_type
order by company_id, name;

set role guest;
select bag_eq(
	'campsite_type_data',
	$$ values (2, 'Wooden lodge')
	        , (4, 'Bungalow')
	$$,
	'Everyone should be able to list all campsite types across all companies'
);
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');

select lives_ok(
	$$ insert into campsite_type(company_id, name) values (2, 'Another type' ) $$,
	'Admin from company 2 should be able to insert a new campsite type to that company.'
);

select bag_eq(
	'campsite_type_data',
	$$ values (2, 'Wooden lodge')
	        , (2, 'Another type')
	        , (4, 'Bungalow')
	$$,
	'The new row should have been added'
);

select lives_ok(
	$$ update campsite_type set name = 'Another' where company_id = 2 and name = 'Another type' $$,
	'Admin from company 2 should be able to update campsite type of that company.'
);

select bag_eq(
	'campsite_type_data',
	$$ values (2, 'Wooden lodge')
	        , (2, 'Another')
	        , (4, 'Bungalow')
	$$,
	'The row should have been updated.'
);

select lives_ok(
	$$ delete from campsite_type where company_id = 2 and name = 'Another' $$,
	'Admin from company 2 should be able to delete campsite type from that company.'
);

select bag_eq(
	'campsite_type_data',
	$$ values (2, 'Wooden lodge')
	        , (4, 'Bungalow')
	$$,
	'The row should have been deleted.'
);

select throws_ok(
	$$ insert into campsite_type (company_id, name) values (4, 'Another type' ) $$,
	'42501', 'new row violates row-level security policy for table "campsite_type"',
	'Admin from company 2 should NOT be able to insert new campsite types to company 4.'
);

select lives_ok(
	$$ update campsite_type set name = 'Nope' where company_id = 4 $$,
	'Admin from company 2 should not be able to update new campsite types of company 4, but no error if company_id is not changed.'
);

select bag_eq(
	'campsite_type_data',
	$$ values (2, 'Wooden lodge')
	        , (4, 'Bungalow')
	$$,
	'No row should have been changed.'
);

select throws_ok(
	$$ update campsite_type set company_id = 4 where company_id = 2 $$,
	'42501', 'new row violates row-level security policy for table "campsite_type"',
	'Admin from company 2 should NOT be able to move campsite types to company 4'
);

select lives_ok(
	$$ delete from campsite_type where company_id = 4 $$,
	'Admin from company 2 should NOT be able to delete campsite types from company 4, but not error is thrown'
);

select bag_eq(
	'campsite_type_data',
	$$ values (2, 'Wooden lodge')
	        , (4, 'Bungalow')
	$$,
	'No row should have been changed'
);

select throws_ok(
	$$ insert into campsite_type (company_id, name) values (2, '  ' ) $$,
	'23514', 'new row for relation "campsite_type" violates check constraint "name_not_empty"',
	'Should not be able to insert campsite types with a blank name.'
);

reset role;


select *
from finish();

rollback;

